# AnimationWithRetainedGraphics.py

from Animation import Animation
import random
from Tkinter import *

#########################################################
# AnimationWithRetainedGraphics class
# with Color, Shape, Rectangle, Oval, and Circle classes
#########################################################

class Color(object):
    def __init__(self, red, green, blue):
        self.red = red
        self.green = green
        self.blue = blue

    @classmethod
    def randomColor(cls):
        r = random.randrange(256)
        g = random.randrange(256)
        b = random.randrange(256)
        return Color(r, g, b)

    def rgbString(self):
        return "#%02x%02x%02x" % (self.red, self.green, self.blue)

class Shape(object):
    def __init__(self, left, top, right, bottom):
        self.left = left
        self.top = top
        self.right = right
        self.bottom = bottom
        self.fill = None
        self.line = Color(0, 0, 0)
        self.lineWidth = 1
    
    def getFill(self):
        return None if (self.fill == None) else self.fill.rgbString()
        
    def getLine(self):
        return None if (self.line == None) else self.line.rgbString()

class Rectangle(Shape):
    def draw(self, canvas):
        canvas.create_rectangle(self.left, self.top, self.right, self.bottom,
                                fill=self.getFill(),
                                outline=self.getLine(), width=self.lineWidth)

class Oval(Shape):       
    def draw(self, canvas):
        canvas.create_oval(self.left, self.top, self.right, self.bottom,
                           fill=self.getFill(),
                           outline=self.getLine(), width=self.lineWidth)

class Circle(Oval):
    def __init__(self, cx, cy, r):
        (left, top, right, bottom) = (cx-r, cy-r, cx+r, cy+r)
        # Call superclass's __init__ method
        super(Circle, self).__init__(left, top, right, bottom)

class AnimationWithRetainedGraphics(Animation):
    def __init__(self):
        self.shapes = [ ]

    def addShape(self, shape):
        self.shapes.append(shape)
        return shape

    def createRectangle(self, left, top, right, bottom):
        return self.addShape(Rectangle(left, top, right, bottom))

    def createOval(self, left, top, right, bottom):
        return self.addShape(Oval(left, top, right, bottom))

    def createCircle(self, cx, cy, r):
        return self.addShape(Circle(cx, cy, r))

    def redrawAll(self):
        for shape in self.shapes:
            shape.draw(self.canvas)
